#ifndef _PATRICIA_H
#define _PATRICIA_H

/* typedef unsigned int u_int; */
typedef void (*void_fn_t)();
#define prefix_touchar(prefix) ((u_char *)&(prefix)->add.sin)
#define BIT_TEST(f, b)  ((f) & (b))


#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>

typedef struct _prefix4_t {
    u_short family;		/* AF_INET | AF_INET6 */
    u_short bitlen;		/* same as mask? */
    int ref_count;		/* reference count */
    struct in_addr sin;
} prefix4_t;

typedef struct _prefix_t {
    u_short family;		/* AF_INET | AF_INET6 */
    u_short bitlen;		/* same as mask? */
    int ref_count;		/* reference count */
    union {
		struct in_addr sin;
#ifdef HAVE_IPV6
		struct in6_addr sin6;
#endif /* IPV6 */
    } add;
} prefix_t;


typedef struct _patricia_node_t {
   u_int bit;			/* flag if this node used */
   prefix_t *prefix;		/* who we are in patricia tree */
   struct _patricia_node_t *l, *r;	/* left and right children */
   struct _patricia_node_t *parent;/* may be used */
   void *data;			/* pointer to data */
   void	*user1;			/* pointer to usr data (ex. route flap info) */
} patricia_node_t;

typedef struct _patricia_tree_t {
   patricia_node_t 	*head;
   u_int		maxbits;	/* for IP, 32 bit addresses */
   int num_active_node;		/* for debug purpose */
   int (*match_func)(patricia_node_t* node); /* can be used to ingore a node, example: expire time or ttl */
} patricia_tree_t;

/* Functions used By Netsweeper */
/* Entry point functions that should be used */
void* patricia_find_exact_data(patricia_tree_t * tree, in_addr_t ip, int cidr);
int patricia_find_exact(patricia_tree_t * tree, in_addr_t ip, int cidr);
void* patricia_find_data(patricia_tree_t * tree, in_addr_t ip);
int patricia_find(patricia_tree_t * tree, in_addr_t ip);
int patricia_add(patricia_tree_t* tree, in_addr_t ip, int cidr, void* data);
void* patricia_delete(patricia_tree_t* tree, in_addr_t ip, int cidr);
patricia_tree_t * patricia_init (int maxbits, int (*match_func)(patricia_node_t* node));
void patricia_terminate(patricia_tree_t* tree, void_fn_t func);
size_t patricia_walk(patricia_tree_t *patricia, void_fn_t func, void* arg);
size_t patricia_walk_inorder(patricia_node_t *node, void_fn_t func, void* arg);

int patricia_ipv6_add(patricia_tree_t* tree, struct in6_addr *ip, int cidr, void* data);
void* patricia_ipv6_delete(patricia_tree_t* tree, struct in6_addr *ip, int cidr);
int patricia_ipv6_find(patricia_tree_t * tree, struct in6_addr *ip);
int patricia_ipv6_find_exact(patricia_tree_t * tree, struct in6_addr *ip, int cidr);
void* patricia_ipv6_find_data(patricia_tree_t * tree, struct in6_addr *ip);
void* patricia_ipv6_find_exact_data(patricia_tree_t * tree, struct in6_addr *ip, int cidr);

#define PATRICIA_MAXBITS 128
#define PATRICIA_NBIT(x)        (0x80 >> ((x) & 0x7f))
#define PATRICIA_NBYTE(x)       ((x) >> 3)

#define PATRICIA_DATA_GET(node, type) (type *)((node)->data)
#define PATRICIA_DATA_SET(node, value) ((node)->data = (void *)(value))

#define PATRICIA_WALK(Xhead, Xnode) \
    do { \
        patricia_node_t *Xstack[PATRICIA_MAXBITS+1]; \
        patricia_node_t **Xsp = Xstack; \
        patricia_node_t *Xrn = (Xhead); \
        while ((Xnode = Xrn)) { \
            if (Xnode->prefix)

#define PATRICIA_WALK_ALL(Xhead, Xnode) \
do { \
        patricia_node_t *Xstack[PATRICIA_MAXBITS+1]; \
        patricia_node_t **Xsp = Xstack; \
        patricia_node_t *Xrn = (Xhead); \
        while ((Xnode = Xrn)) { \
	    if (1)

#define PATRICIA_WALK_BREAK { \
	    if (Xsp != Xstack) { \
		Xrn = *(--Xsp); \
	     } else { \
		Xrn = (patricia_node_t *) 0; \
	    } \
	    continue; }

#define PATRICIA_WALK_END \
            if (Xrn->l) { \
                if (Xrn->r) { \
                    *Xsp++ = Xrn->r; \
                } \
                Xrn = Xrn->l; \
            } else if (Xrn->r) { \
                Xrn = Xrn->r; \
            } else if (Xsp != Xstack) { \
                Xrn = *(--Xsp); \
            } else { \
                Xrn = (patricia_node_t *) 0; \
            } \
        } \
    } while (0)

#endif /* _PATRICIA_H */
