/**
 * @file
 * @brief Declarations of recognized rewriter protocols
 *
 * This module contains the algorithms for parsing, validating,
 * and encoding specific protocols
 *
 * These protocol definitions are used by protocol.c.
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 **/

#ifndef PROTOCOL_DEF_H_INCLUDE
#define PROTOCOL_DEF_H_INCLUDE

#include "protocol.h"

/* Any time you add a protocol, you must define these functions for that protocol. */
/* parser functions return 0 on valid parse and -1 on invalid parse.
 *   errno EIO - parse error
 *   errno ENOMEM - memory error */
typedef int (protocol_parser_func)(protocol_input_line *);
typedef int (protocol_encoder_func)(GlobalConf *conf, const protocol_input_line *, char *, size_t);

protocol_parser_func squid24_parse;
protocol_encoder_func squid24_encode;

protocol_parser_func squid25_parse;
protocol_encoder_func squid25_encode;

protocol_parser_func squid26_parse;
protocol_encoder_func squid26_encode;

protocol_parser_func squid27_parse;
protocol_encoder_func squid27_encode;

protocol_parser_func squid30_parse;
protocol_encoder_func squid30_encode;

protocol_parser_func squid31_parse;
protocol_encoder_func squid31_encode;

protocol_parser_func squid32_parse;
protocol_encoder_func squid32_encode;

#endif
