/**
 * @file
 * @brief The reference implementation of the socket server
 *
 * This will stay alive for 10 seconds, then stop
 * Also any signal will stop the operation in about 1 second
 *
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>
#include "socketserver.h"

/** @brief Global most amazing server object, we need a method to "shutdown" */
static int running = 1;

void stop(__attribute__((unused)) int signal)
{
	running = 0;
}

void connect_callback(struct client_socket* s)
{
	printf("%p: Connected\n", s);
}
void recv_callback(struct client_socket* s)
{
	printf("%p: Data %s\n", s, s->rxbuffer);
	// echo it back
	socketserver_send(s, s->rxbuffer, s->rxlen);
}
void disconnect_callback(struct client_socket* s)
{
	printf("%p: Disconnect\n", s);
}

int main()
{
	struct socketserver* svr;

	signal(SIGTERM, stop);
	signal(SIGQUIT, stop);
	signal(SIGINT, stop);
	signal(SIGPIPE,SIG_IGN);

	// Create a new amazing server
	svr = socketserver_create(12345, connect_callback, recv_callback, disconnect_callback);

	// Perform your test here
	// this test we simply stay alive for 10 seconds
	while ( running != 0 && running++ < 10 ) {
		sleep(1);
	}
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);

	return 0;
}
